@@ -9,21 +9,20 @@ set -eux
 OVB_CREDS_FILE=$1
 PLAYBOOK=$2
 
-# env file is named <prefix>env.yaml
-# prefix is built from:
-# "{{ 1000 |random }}"-"{{ lookup('env', 'USER') }}"-"{{ lookup('env', 'BUILD_NUMBER') }}"-
+# env file is named env-{{ idnum }}.yaml
+# idnum is built from:
+# "{{ 100000 |random }}"
 
-export PREFIX=$(ls $WORKSPACE | grep -h env.yaml | sed -n -e 's/env.yaml//p')
-echo $PREFIX
+export IDNUM=$(ls $WORKSPACE | grep -h 'env-.*\.yaml' | sed -e 's/env-\(.*\).yaml/\1/')
+echo $IDNUM
 
 pushd $WORKSPACE/tripleo-quickstart
 
 bash quickstart.sh \
 --bootstrap \
 --working-dir $WORKSPACE/ \
---extra-vars prefix=$PREFIX \
+--extra-vars idnum=$IDNUM \
 --extra-vars @$OVB_CREDS_FILE \
 --playbook $PLAYBOOK \
 localhost
 popd
-
