@@ -23,13 +23,17 @@ $WORKSPACE/bin/cico node get \
     --count 1 \
     --retry-count 6 \
     --retry-interval 60 \
-    -f csv > $WORKSPACE/provisioned.csv
+    -f csv | sed "1d" > $WORKSPACE/provisioned.csv
 
 $WORKSPACE/bin/cico inventory
-cat $WORKSPACE/provisioned.csv
+if [ -s $WORKSPACE/provisioned.csv ]; then
+    cat $WORKSPACE/provisioned.csv
+else
+    echo "FATAL: no nodes were provisioned"
+    exit 1
+fi
 
 export VIRTHOST=`cat provisioned.csv | tail -1 | cut -d "," -f 3| sed -e 's/"//g'`
 export VIRTHOST_KEY=`cat provisioned.csv | tail -1 | cut -d "," -f 7| sed -e 's/"//g'`
 echo $VIRTHOST > $WORKSPACE/virthost
 echo $VIRTHOST_KEY > $WORKSPACE/cico_key.txt
-
