@@ -1,6 +1,6 @@
 #!/bin/bash
 
-DEFAULT_OPT_TAGS="untagged,provision,environment,undercloud-scripts,overcloud-scripts"
+DEFAULT_OPT_TAGS="untagged,provision,environment,undercloud-inventory,overcloud-scripts"
 
 : ${OPT_BOOTSTRAP:=0}
 : ${OPT_SYSTEM_PACKAGES:=0}
@@ -87,7 +87,7 @@ bootstrap () {
     if [ "$OPT_NO_CLONE" != 1 ]; then
         if ! [ -d "$OOOQ_DIR" ]; then
             echo "Cloning tripleo-quickstart repository..."
-            git clone https://github.com/openstack/tripleo-quickstart.git \
+            git clone https://git.openstack.org/openstack/tripleo-quickstart \
                 $OOOQ_DIR
         fi
 
@@ -283,7 +283,7 @@ RELEASE=$2
 # fact.
 if [ -z "$RELEASE" ]; then
 
-    RELEASE=mitaka
+    RELEASE=rocky
 
 fi
 
@@ -366,4 +366,3 @@ Virtual Environment Setup Complete
 ##################################
 EOF
 fi
-
